<?php
namespace MOD\Controller;

if ( ! function_exists( 'add_action' ) ) {
	exit( 0 );
}

use MOD\MOD_Core as Core;
use MOD\View\MOD_Options as Options_View;

class MOD_Options {
	public function __construct() {
		add_filter( 'plugin_action_links_mod-auto-updates/mod-auto-updates.php', array( $this, 'plugin_links' ) );
		add_action( 'in_plugin_update_message-mod-auto-updates/mod-auto-updates.php', array( $this, 'mod_plugin_update_message' ), 10, 2 );
		add_filter( 'cron_schedules', array( $this, 'mod_add_cron_interval' ) );
		add_action( 'admin_init', array( $this, 'mod_register_schedule' ) );
		add_action( 'admin_notices', array( $this, 'mod_settings_message' ) );
	}
    /**
	 * Add link settings page
	 *
	 * @since 1.0
	 * @param Array $links
	 * @return Array
	 */
	public function plugin_links( $links ) {
		$links_settings = array( sprintf(
            '<a href="%s">%s</a>',
			'options-general.php?page=mod-auto-updates',
            __( 'Configurações', Core::TEXTDOMAIN )
		) );

		$support_settings = array( sprintf(
            '<a href="%s">%s</a>',
			'https://updates.mercadoonlinedigital.com/suporte-mod/',
            __( 'Suporte MOD', Core::TEXTDOMAIN )
		) );

		return array_merge( $links_settings, $support_settings, $links );
	}
  	/**
	 * Change message update
	 *
	 * @since 1.0.8
	 * @param String $links
	 * @return String
	 */
	public function mod_plugin_update_message( $data, $response ) {
		Options_View::mod_update_plugin_html( $data );
	}

	public function mod_add_cron_interval( $schedules ) {
		$schedules['mod_tmp_plugins'] = array(
			'interval' => 10 * HOUR_IN_SECONDS,
			'display'  => esc_html__( 'Consulta a cada 10 Horas' )
		);

		$schedules['mod_tmp_themes'] = array(
			'interval' => 10 * HOUR_IN_SECONDS,
			'display'  => esc_html__( 'Consulta a cada 10 Horas' )
		);

		return $schedules;
	}

	public function mod_register_schedule() {
		if ( ! wp_next_scheduled( 'mod_remove_plugins' ) ) {
			wp_schedule_event( time(), 'mod_tmp_plugins', 'mod_remove_plugins' );
		}

		if ( ! wp_next_scheduled( 'mod_remove_themes' ) ) {
			wp_schedule_event( time(), 'mod_tmp_themes', 'mod_remove_themes' );
		}
	}
	/**
	 * Admin Messages
	 *
	 * @since 1.0
	 * @param Array $links
	 * @return Array
	 */
	public function mod_settings_message() {
		Options_View::mod_settings_message_html();
	}
}
