<?php
namespace MOD\Model;

if ( ! function_exists( 'add_action' ) ) {
	exit( 0 );
}

use MOD\MOD_Core as Core;
use MOD\Helper\MOD_Utils as Utils;

class MOD_Setting {
	public function mod_page_register_fields( $page_name ) {
		register_setting(
            'mod_auto_updates_group',
            'mod_auto_updates_field',
            [ $this, 'sanitize' ]
        );

        add_settings_section(
            'setting_section_id',
            __( 'MOD Atualizações Autómaticas', Core::TEXTDOMAIN ),
            [ $this, 'print_section_info' ],
            $page_name
        );

        add_settings_field(
            'mod_token_number',
			__( 'Token', Core::TEXTDOMAIN ),
            [ $this, 'mod_token_callback' ],
            $page_name,
            'setting_section_id'
        );

        add_settings_field(
            'mod_log_settings',
			__( 'Logs', Core::TEXTDOMAIN ),
            [ $this, 'mod_log_callback' ],
            $page_name,
            'setting_section_id'
        );
	}
	/**
     * Sanitize each setting field as needed
     *
     * @param array $input Contains all settings fields as array keys
     */
    public function sanitize( $input ) {
        $new_input = [];

        if ( isset( $input['mod_token_number'] ) ) {
            $new_input['mod_token_number'] = sanitize_text_field( $input['mod_token_number'] );
        }

        if ( isset( $input['mod_log_settings'] ) ) {
            $new_input['mod_log_settings'] = sanitize_text_field( $input['mod_log_settings'] );
        }

        return $new_input;
	}
	/**
     * Print the Section text
    */
    public function print_section_info() {
        printf( '<p>%s</p>
            <p>%s <b>%s</b></p>
            <p>%s <b>%s</b></p>',
            __( 'Para funcionamento das atualizações necessário inserir o token.', Core::TEXTDOMAIN ),
            __( 'O plugin está configurado para procurar por atualizações a', Core::TEXTDOMAIN ),
            __( 'CADA 10 HORAS.', Core::TEXTDOMAIN ),
            __( 'Para procurar manualmente clique no submenu', Core::TEXTDOMAIN ),
            __( 'LOGS.', Core::TEXTDOMAIN )
        );
    }
    /**
     * Get the settings option array and print one of its values
     */
    public function mod_token_callback() {
		$options = get_option( 'mod_auto_updates_field' );

        printf(
			'<input type="password" id="mod_token_number" name="mod_auto_updates_field[mod_token_number]" value="%s" required/>
			<p class="description" id="tagline-description">%s</p>',
			isset( $options['mod_token_number'] ) ? esc_attr( $options['mod_token_number']) : '',
			__( 'Campo para inserir seu Token para atualização dos plugins e/ou temas.', Core::TEXTDOMAIN )
        );
    }
    /**
     * Get the settings option array and print one of its values
    */
    public function mod_log_callback() {
        printf(
            '<p class="description" id="tagline-description">%s <a href="%s" title="%s">%s.</a></p>',
            __( 'Para visualizar se possui atualização disponível, clique ', Core::TEXTDOMAIN ),
            Utils::get_admin_url( 'admin.php?page=mod-auto-updates-log' ),
            __( 'Página de Log ', Core::TEXTDOMAIN ),
            __( 'AQUI', Core::TEXTDOMAIN )
        );
    }
}
