<?php
/**
 * User Registration Content Restriction Meta Boxes
 *
 * Sets up the write panels used by custom post types.
 *
 * @class    URCR_Admin_Meta_Boxes
 * @version  1.0.0
 * @package  UserRegistrationContentRestriction/Admin/Meta Boxes
 * @category Admin
 * @author   WPEverest
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * URCR_Admin_Meta_Boxes Class
 */

class URCR_Admin_Meta_Box {

    /**
     * Constructor.
     */

    public function __construct() {
        $this->id    = 'content_restriction';

        if(get_option('user_registration_content_restriction_enable') == "no"){
            return;
        }

        if ( is_admin()) {
            add_action( 'load-post.php',     array( $this, 'init_metabox' ) );
            add_action( 'load-post-new.php', array( $this, 'init_metabox' ) );
            add_action( 'admin_enqueue_scripts', array($this, 'register_scripts'));
            add_action( 'admin_enqueue_scripts', array( $this, 'register_styles' ) );
        }

    }

    public function register_scripts()
    {
        $suffix    = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
        wp_register_script( 'select2-js', URCR()->plugin_url() . '/assets/select2/js/select2.full' . $suffix . '.js', array( 'jquery' ), '3.5.4' );
        wp_register_script( 'custom-js', URCR()->plugin_url() . '/assets/custom' . $suffix . '.js', array( 'jquery' ), '3.5.4' );
    }

    public function register_styles()
    {
        wp_register_style ( 'select2-css', URCR()->plugin_url() . '/assets/select2/css/select2.css', array(), URCR_VERSION );
        wp_register_style ( 'user-registration-content-restriction-metabox-css', URCR()->plugin_url() . '/assets/css/ur-content-restriction-metabox-style.css', array(), URCR_VERSION );

        wp_enqueue_style('user-registration-content-restriction-metabox-css');
    }

    /**
     * Meta box initialization.
     */
    public function init_metabox() {
        add_action( 'add_meta_boxes', array( $this, 'add_metabox'  )        );
        add_action( 'save_post',      array( $this, 'save_metabox' ), 10, 2 );
    }

    /**
     * Adds the meta box.
     */
    public function add_metabox() {
        add_meta_box(
            'urcr-meta-box',
            __( 'Restrict This Content', 'user-registration-content-restriction' ),
            array( $this, 'render_metabox' ),
            $screen=null,
            'advanced',
            'default'
        );
    }

    /**
     * Renders the meta box.
     */
    public function render_metabox( $post ) {

        echo "<p>" .__('Use shortcode [urcr_restrict]....[/urcr_restrict] to restrict partial contents.','user-registration-content-restriction') . "</p>";
        $this->content_restriction_metabox_checkbox( array(
                'id'    => 'user_registration_individual_content_restriction_checkbox',
                'label' => 'Restrict Full Content: ',
                'type'  => 'Checkbox',
        ));

        $this->content_restriction_metabox_override_global_settings_checkbox( array(
                'id'    => 'user_registration_individual_content_restriction_override_global_settings',
                'label' => 'Override Global Settings ',
                'type'  => 'Checkbox',
        ));

        $this->content_restriction_metabox_select( array(
                'id'       => 'user_registration_individual_content_restriction_allow_to',
                'label'    => __( 'Allow Access To: ', 'user-registration-content-restriction' ),
                'options'  => array('All Logged In Users','Choose Specific Roles','Guest Users'),
                'desc'     => __( 'Only select this if you want to override global setting for allow option', 'user-registration-content-restriction' ),
        ));

        $this->content_restriction_metabox_multiple_select( array(
                'id'       => 'user_registration_individual_content_restriction_roles[]',
                'label'    => __( 'Allow Access To Roles: ', 'user-registration-content-restriction' ),
                'options'  => urcr_get_all_roles(),
                'desc'     => __( 'Only select this if you want to override global setting for access roles', 'user-registration-content-restriction' ),
        ));

        do_action('render_metabox_complete');
    }

    public function content_restriction_metabox_override_global_settings_checkbox($field)
    {
        global $thepostid, $post;

        $get_meta_data = get_post_meta($post->ID,'urcr_meta_override_global_settings', $single = true);

        $thepostid              = empty( $thepostid ) ? $post->ID : $thepostid;
        $field['class']         = isset( $field['class'] ) ? $field['class'] : 'urcr-override-global-settings-checkbox';
        $field['style']         = isset( $field['style'] ) ? $field['style'] : '';
        $field['wrapper_class'] = isset( $field['wrapper_class'] ) ? $field['wrapper_class'] : '';
        $field['value']         = isset( $field['value'] ) ? $field['value'] : get_post_meta( $thepostid, $field['id'], true );
        $field['name']          = isset( $field['name'] ) ? $field['name'] : $field['id'];

        echo '<p class="form-field ' . esc_attr( $field['id'] ) . '_field ' . esc_attr( $field['wrapper_class'] ) . '"><label for="' . esc_attr( $field['id'] ) . '">' . wp_kses_post( $field['label'] ) . '</label>';

        $non_checked = '<input type="checkbox" id="' . esc_attr( $field['id'] ) . '" name="' . esc_attr( $field['name'] ) . '" class="' . esc_attr( $field['class'] ) . '" style="' . esc_attr( $field['style'] ) . '" >';

        $checked = '<input type="checkbox" id="' . esc_attr( $field['id'] ) . '" name="' . esc_attr( $field['name'] ) . '" class="' . esc_attr( $field['class'] ) . '" style="' . esc_attr( $field['style'] ) . '" checked>';

        if($get_meta_data == "on")
        {
            echo $checked;
        }
        else
        {
            echo $non_checked;
        }
    }

    public function content_restriction_metabox_checkbox($field)
    {

        global $thepostid, $post;

        $get_meta_data = get_post_meta($post->ID,'urcr_meta_checkbox', $single = true);

        $thepostid              = empty( $thepostid ) ? $post->ID : $thepostid;
        $field['class']         = isset( $field['class'] ) ? $field['class'] : 'urcr-checkbox';
        $field['style']         = isset( $field['style'] ) ? $field['style'] : '';
        $field['wrapper_class'] = isset( $field['wrapper_class'] ) ? $field['wrapper_class'] : '';
        $field['value']         = isset( $field['value'] ) ? $field['value'] : get_post_meta( $thepostid, $field['id'], true );
        $field['name']          = isset( $field['name'] ) ? $field['name'] : $field['id'];

        echo '<p class="form-field ' . esc_attr( $field['id'] ) . '_field ' . esc_attr( $field['wrapper_class'] ) . '"><label for="' . esc_attr( $field['id'] ) . '">' . wp_kses_post( $field['label'] ) . '</label>';

        $non_checked = '<input type="checkbox" id="' . esc_attr( $field['id'] ) . '" name="' . esc_attr( $field['name'] ) . '" class="' . esc_attr( $field['class'] ) . '" style="' . esc_attr( $field['style'] ) . '" >';

        $checked = '<input type="checkbox" id="' . esc_attr( $field['id'] ) . '" name="' . esc_attr( $field['name'] ) . '" class="' . esc_attr( $field['class'] ) . '" style="' . esc_attr( $field['style'] ) . '" checked>';

        if($get_meta_data == "on")
        {
            echo $checked;
        }
        else
        {
            echo $non_checked;
        }
    }
    
    public function content_restriction_metabox_select( $field ) {

        wp_enqueue_script('select2-js');
        wp_enqueue_script('custom-js');

        wp_enqueue_style('select2-css');

        global $thepostid, $post;

        $thepostid              = empty( $thepostid ) ? $post->ID : $thepostid;
        $field['class']         = isset( $field['class'] ) ? $field['class'] : 'select';
        $field['style']         = isset( $field['style'] ) ? $field['style'] : '';
        $field['wrapper_class'] = isset( $field['wrapper_class'] ) ? $field['wrapper_class'] : '';
        $field['value']         = isset( $field['value'] ) ? $field['value'] : get_post_meta( $thepostid, $field['id'], true );
        $field['name']          = isset( $field['name'] ) ? $field['name'] : $field['id'];

        $get_meta_data = get_post_meta($post->ID,'urcr_allow_to', $single = true);

        echo '<div class="urcr-select">';
        echo '<p class="form-field ' . esc_attr( $field['id'] ) . '_field ' . esc_attr( $field['wrapper_class'] ) . '"><label for="' . esc_attr( $field['id'] ) . '">' . wp_kses_post( $field['label'] ) . '</label><select id="' . esc_attr( $field['id'] ) . '" name="' . esc_attr( $field['name'] ) . '" class="' . esc_attr( $field['class'] ) . '" style="' . esc_attr( $field['style'] ) . '" >';


        foreach ( $field['options'] as $key => $value ) {
            ?>
                <option value="<?php echo esc_attr( $key ); ?>" <?php

                if ( is_array( $get_meta_data ) ) {
                    selected( in_array( $key, $get_meta_data ), true );
                } else {
                    selected( $get_meta_data, $key );
                }

                    ?>><?php echo $value ?></option>
            <?php
            }


        echo '</select> ';
        echo '</p>';
        echo '</div>';
    }

    public function content_restriction_metabox_multiple_select( $field ) {

        wp_enqueue_script('select2-js');
        wp_enqueue_script('custom-js');

        wp_enqueue_style('select2-css');

        global $thepostid, $post;

        $thepostid              = empty( $thepostid ) ? $post->ID : $thepostid;
        $field['class']         = isset( $field['class'] ) ? $field['class'] : 'multiple-select';
        $field['style']         = isset( $field['style'] ) ? $field['style'] : '';
        $field['wrapper_class'] = isset( $field['wrapper_class'] ) ? $field['wrapper_class'] : '';
        $field['value']         = isset( $field['value'] ) ? $field['value'] : get_post_meta( $thepostid, $field['id'], true );
        $field['name']          = isset( $field['name'] ) ? $field['name'] : $field['id'];

        $get_meta_data = get_post_meta($post->ID,'urcr_meta_roles', $single = true);

        echo '<div class="urcr-multiple-select">';
        echo '<p class="form-field ' . esc_attr( $field['id'] ) . '_field ' . esc_attr( $field['wrapper_class'] ) . '"><label for="' . esc_attr( $field['id'] ) . '">' . wp_kses_post( $field['label'] ) . '</label><select multiple id="' . esc_attr( $field['id'] ) . '" name="' . esc_attr( $field['name'] ) . '" class="' . esc_attr( $field['class'] ) . '" style="' . esc_attr( $field['style'] ) . '" >';


            foreach ( $field['options'] as $key => $value ) {
            ?>
                <option value="<?php echo esc_attr( $key ); ?>" <?php

                if ( is_array( $get_meta_data ) ) {
                    selected( in_array( $key, $get_meta_data ), true );
                } else {
                    selected( $get_meta_data, $key );
                }

                    ?>><?php echo $value ?></option>
            <?php
            }

        echo '</select> ';
        echo '</p>';
        echo '</div>';
    }

    /**
     * Handles saving the meta box.
     *
     * @param int     $post_id Post ID.
     * @param WP_Post $post    Post object.
     * @return null
     */
    public function save_metabox( $post_id, $post ) {

        if ( empty( $_POST ) ) {
            return false;
        }

        $checkbox = isset($_POST['user_registration_individual_content_restriction_checkbox'])?$_POST['user_registration_individual_content_restriction_checkbox']:'';

        $override_global_settings = isset($_POST['user_registration_individual_content_restriction_override_global_settings'])?$_POST['user_registration_individual_content_restriction_override_global_settings']:'';

        $allow_to = isset($_POST['user_registration_individual_content_restriction_allow_to'])?$_POST['user_registration_individual_content_restriction_allow_to']:'';

        $array_of_roles = isset($_POST['user_registration_individual_content_restriction_roles'])?$_POST['user_registration_individual_content_restriction_roles']:'';

        update_post_meta( $post_id, 'urcr_meta_checkbox', $checkbox);

        update_post_meta( $post_id, 'urcr_meta_override_global_settings', $override_global_settings);
        
        update_post_meta( $post_id, 'urcr_allow_to', $allow_to);

        update_post_meta( $post_id, 'urcr_meta_roles', $array_of_roles);

        // Add nonce for security and authentication.
        $nonce_name   = isset( $_POST['custom_nonce'] ) ? $_POST['custom_nonce'] : '';
        $nonce_action = 'custom_nonce_action';

        // Check if nonce is set.
        if ( ! isset( $nonce_name ) ) {
            return;
        }

        // Check if nonce is valid.
        if ( ! wp_verify_nonce( $nonce_name, $nonce_action ) ) {
            return;
        }

        // Check if user has permissions to save data.
        if ( ! current_user_can( 'edit_post', $post_id ) ) {
            return;
        }

        // Check if not an autosave.
        if ( wp_is_post_autosave( $post_id ) ) {
            return;
        }

        // Check if not a revision.
        if ( wp_is_post_revision( $post_id ) ) {
            return;
        }
    }
}

new URCR_Admin_Meta_Box();