<?php
/**
 * User Registration Invite Codes Settings class
 * @version   1.0.0
 */
if( ! defined ( 'ABSPATH' ) ){
	exit;
}

if( ! class_exists( 'UR_Invite_Codes_Settings' ) ) :

	/**
	 * UR_Invite_Codes_Settings Setting.
	 */
	class UR_Invite_Codes_Settings extends UR_Settings_Page {

		/**
		 * Constructor.
		 */
		public function __construct() {
			$this->id = 'user-registration-invite-codes';
			$this->label = esc_html__( 'Invite Codes', 'user-registration-invite-codes' );

			add_filter( 'user_registration_settings_tabs_array', array( $this, 'add_settings_page' ), 20 );
			add_action( 'user_registration_sections_' . $this->id, array( $this, 'output_sections' ) );
			add_action( 'user_registration_settings_' . $this->id, array( $this, 'output' ) );
			add_filter( 'show_user_registration_setting_message', array( $this, 'filter_notice' ) );
			add_filter( 'user-registration-setting-save-label', array( $this, 'filter_label' ) );
			add_filter( 'user_registration_settings_save_action', array( $this, 'filter_save_action' ) );
		}

		/**
		 * Get sections.
		 *
		 * @return array
		 */
		public function get_sections() {
			$sections = array(
				''		=> __( 'Codes List', 'user-registration-invite-codes' ),
				'add-new'        	=> __( 'Add New', 'user-registration-invite-codes' ),
				'generate-codes'    => __( 'Generate Random Codes', 'user-registration-invite-codes' ),
				'raw-list' 			=> __( 'Raw List', 'user-registration-invite-codes' ),
				'edit-code'			=> __( '', 'user-registration-invite-codes' )
				);

			return apply_filters( 'user_registration_get_sections_' . $this->id, $sections );
		}

		/**
		 * Get Add New Code Settings.
		 * @return array.
		 */
		public function get_add_new_code_settings() {

			$settings = apply_filters(
				'user_registration_get_add_new_code_settings', array(

					array(
						'title' => __( 'Add New Code', 'user-registration-invite-codes' ),
						'type'  => 'title',
						'desc'  => '',
						'id'    => 'add_new_code_options',
					),
          			array(
						'title'    => __( 'Add New Code', 'user-registration-invite-codes' ),
						'desc'     => __( 'Add new code to enable users register with this code.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_add_new_code',
						'type'     => 'text',
						'css'      => 'min-width: 350px;',
						'desc_tip' => true,
					),
					array(
						'title'    => __( 'User Limit', 'user-registration-invite-codes' ),
						'desc'     => __( 'Limit the number of users can use this code. Leave empty for unlimited times.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_user_limit',
						'type'     => 'number',
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
					),
					array(
						'title'    => __( 'Expiry Date', 'user-registration-invite-codes' ),
						'desc'     => __( 'Set the date in which the code will be expired. Leave empty for no expiry date', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_expiry_date',
						'type'     => 'date',
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
					),
					array(
						'title'    => __( 'Select Form', 'user-registration-invite-codes' ),
						'desc'     => __( 'Select the form to set this invite code to succesfully register.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_select_form',
						'type'     => 'select',
						'options'  => ur_get_all_user_registration_form(),
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
					),
					array(
						'title'    => __( 'User Role', 'user-registration-invite-codes' ),
						'desc'     => __( 'Override the user role from form settings and assign to the user registered with this invite code.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_user_role',
						'type'     => 'select',
						'default'  => 'subscriber',
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
						'display'  => 'none',
						'options'  => ur_get_default_admin_roles(),
					),

					array(
						'type' => 'sectionend',
						'id'   => 'add_new_code_options',
					)
			));

			return apply_filters( 'user_registration_get_add_new_code_settings_' . $this->id, $settings );
		}

		/**
		 * Generate random code settings.
		 * @return array.
		 */
		public function get_random_codes_settings() {

			$settings = apply_filters(
				'user_registration_get_random_codes_settings', array(

					array(
						'title' => __( 'Generate Random Codes', 'user-registration-invite-codes' ),
						'type'  => 'title',
						'desc'  => '',
						'id'    => 'generate_random_code_options',
					),
					array(
						'title'    => __( 'Code numbers', 'user-registration-invite-codes' ),
						'desc'     => __( 'Number of random codes to generate.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_random_number',
						'type'     => 'number',
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
					),
          			array(
						'title'    => __( 'Code Prefix', 'user-registration-invite-codes' ),
						'desc'     => __( 'Add prefix to randomly generated codes.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_random_codes_prefix',
						'type'     => 'text',
						'css'      => 'min-width: 350px;',
						'desc_tip' => true,
					),
					array(
						'title'    => __( 'User Limit', 'user-registration-invite-codes' ),
						'desc'     => __( 'Limit the number of users can use these code.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_random_user_limit',
						'type'     => 'number',
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
					),
					array(
						'title'    => __( 'Expiry Date', 'user-registration-invite-codes' ),
						'desc'     => __( 'Set the date in which these codes will be expired. Leave empty for no expiry date', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_random_expiry_date',
						'type'     => 'date',
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
					),
					array(
						'title'    => __( 'Select Form', 'user-registration-invite-codes' ),
						'desc'     => __( 'Select the form to set this invite code to succesfully register.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_random_select_form',
						'type'     => 'select',
						'options'  => ur_get_all_user_registration_form(),
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
					),
					array(
						'title'    => __( 'User Role', 'user-registration-invite-codes' ),
						'desc'     => __( 'Override the user role from form settings and assign to the user registered with this invite code.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_random_user_role',
						'type'     => 'select',
						'desc_tip' => true,
						'default'  => 'subscriber',
						'css'      => 'min-width: 350px;',
						'display'  => 'none',
						'options'  => ur_get_default_admin_roles(),
					),

					array(
						'type' => 'sectionend',
						'id'   => 'generate_random_code_options',
					)
			));

			return apply_filters( 'user_registration_get_random_settings_' . $this->id, $settings );
		}

		/**
		 * Edit code settings.
		 * @return array.
		 */
		public function get_edit_code_settings( $editing_code ) {

			$editing_code    = isset( $_POST['user_registration_edit_code'] ) ? $_POST['user_registration_edit_code'] : $editing_code;
			$all_codes 		 = get_option( 'ur_invite_codes', array() );
			$code_options 	 = array();

			foreach( $all_codes as $code => $options ) {

				if( $editing_code === $code ) {
					$code_options = $options;
					break;
				}
			}

			$settings = apply_filters(
				'user_registration_get_edit_code_settings', array(

					array(
						'title' => __( 'Editing Code: ', 'user-registration-invite-codes' ). $editing_code,
						'type'  => 'title',
						'desc'  => '',
						'id'    => 'edit_code_options',
					),
					array(
						'title'    => __( 'Code', 'user-registration-invite-codes' ),
						'desc'     => __( 'Edit code to enable users register with this code.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_edit_code',
						'type'     => 'text',
						'default'  => $editing_code,
						'css'      => 'min-width: 350px;',
						'desc_tip' => true,
					),
					array(
						'title'    => __( 'User Limit', 'user-registration-invite-codes' ),
						'desc'     => __( 'Limit the number of users can use this code. Leave empty for unlimited times.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_user_limit',
						'type'     => 'number',
						'default'  => isset( $options['user_limit'] ) ? $options['user_limit'] : '' ,
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
					),
					array(
						'title'    => __( 'Expiry Date', 'user-registration-invite-codes' ),
						'desc'     => __( 'Set the date in which the code will be expired. Leave empty for no expiry date', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_expiry_date',
						'default'  => isset( $options['expiry_date'] ) ? $options['expiry_date'] : '' ,
						'type'     => 'date',
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
					),
					array(
						'title'    => __( 'Select Form', 'user-registration-invite-codes' ),
						'desc'     => __( 'Select the form to set this invite code to succesfully register.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_select_form',
						'type'     => 'select',
						'default'  =>  isset( $options['form'] ) ? $options['form'] : 0,
						'options'  => ur_get_all_user_registration_form(),
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
					),
					array(
						'title'    => __( 'User Role', 'user-registration-invite-codes' ),
						'desc'     => __( 'Override the user role from form settings and assign to the user registered with this invite code.', 'user-registration-invite-codes' ),
						'id'       => 'user_registration_invite_codes_user_role',
						'type'     => 'select',
						'default'  =>  isset( $options['role'] ) ? $options['role'] : 'subscriber',
						'desc_tip' => true,
						'css'      => 'min-width: 350px;',
						'display'  => 'none',
						'options'  => ur_get_default_admin_roles(),
					),

					array(
						'type' => 'sectionend',
						'id'   => 'edit_code_options',
					)
			));

			return apply_filters( 'user_registration_get_add_new_code_settings_' . $this->id, $settings );
		}

		/**
		 * Outputs Invite Codes Settings page.
		 * @return void
		 */
		public function output() {

			global $current_section;
			global $invite_codes_table_list;

			switch( $current_section ) {
				case '':

					$GLOBALS['hide_save_button'] = true;

					// Add screen option.
					add_screen_option( 'per_page', array(
						'default' => 20,
						'option'  => 'ur_invite_codes_per_page',
					) );

					User_Registration_Invite_Codes_Admin::invite_codes_list_table_output();
				break;

				case 'add-new':

				$handle_action = ur_invite_codes_settings_handler();

				if( $handle_action === true ) {
					echo '<div id="message" class="updated inline notice notice-error"><p><strong>'. __( 'Code successfully generated.', 'user-registration-invite-codes' ) .'</strong></p></div>';
				}

				$settings = $this->get_add_new_code_settings();
				UR_Admin_Settings::output_fields( $settings );

					return;
				break;

				case 'generate-codes':
				$handle_action = ur_invite_codes_settings_handler();

				if( $handle_action === true ) {
					echo '<div id="message" class="updated inline notice notice-error"><p><strong>'. __( 'Random codes successfully generated.', 'user-registration-invite-codes' ) .'</strong></p></div>';
				}
					$settings = $this->get_random_codes_settings();
					UR_Admin_Settings::output_fields( $settings );

					return;
				break;

				case 'raw-list':
					$GLOBALS['hide_save_button'] = true;
					$settings = array(
						array(
							'title' => __( 'Raw List', 'user-registration-invite-codes' ),
							'type'  => 'title',
							'desc'  => 'Raw list of all codes',
							'id'    => 'raw_list_options',
						),

						array(
							'type' => 'sectionend',
							'id'   => 'raw_list_options',
						)
					);

					UR_Admin_Settings::output_fields( $settings );

					$codes 		= get_option( 'ur_invite_codes', array() );
					$codes_list = array_keys( $codes );
					$codes_raw	= implode( "\n", $codes_list );
					$codes 		= is_array( $codes ) ? $codes_raw : __( 'No Codes Yet!', 'user-registration-invite-codes' );

					echo '<textarea cols="40" rows="10">'. $codes .' </textarea>';
				break;


				case 'edit-code':

				if( ! isset( $_REQUEST['editing'] ) ) {
					return;
				}

				$code 		   = $_REQUEST['editing'];
				$handle_action = ur_invite_codes_settings_handler();

				if( $handle_action === true ) {
					echo '<div id="message" class="updated inline notice notice-error"><p><strong>'. __( 'Code successfully updated.', 'user-registration-invite-codes' ) .'</strong></p></div>';
				}
					$settings = $this->get_edit_code_settings( $code );
					UR_Admin_Settings::output_fields( $settings );

					return;
				break;
			}
		}

		/**
		 * Filter Notice for invite codes tab.
		 * @return bool
		 */
		public function filter_notice() {
			global $current_tab;

			if( 'user-registration-invite-codes' === $current_tab ) {
				return false;
			}

			return true;
		}

		/**
		 * Filter submit button label for certain tabs and sections.
		 * @param  string $label Label
		 * @return string        Label
		 */
		public function filter_label( $label ) {
			global $current_tab;
			global $current_section;

			if( 'user-registration-invite-codes' === $current_tab  ) {

			   if( 'add-new' === $current_section ) {
					return __( 'Add Code', 'user-registration-invite-codes' );
				} elseif( 'generate-codes' === $current_section ) {
					return __( 'Generate Codes', 'user-registration-invite-codes' );
				} elseif( 'edit-code' === $current_section ) {
					return __( 'Update Code', 'user-registration-invite-codes' );
				}
			}

			return $label;
		}

		/**
		 * Filters save action. Donot perform save on this tab.
		 *
		 * @return void
		 */
		public function filter_save_action() {
			global $current_tab;
			if( 'user-registration-invite-codes' === $current_tab  ) {
				return false;
			}

			return true;
		}

	}

endif;

return new UR_Invite_Codes_Settings ();
