<?php
/**
 * User_Registration_Invite_Codes_Admin
 *
 * @package  User_Registration_Invite_Codes_Admin
 * @since  1.0.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * @class User_Registration_Invite_Codes_Admin
 */
class User_Registration_Invite_Codes_Admin {

	/**
	 * User_Registration_Invite_Codes_Admin Constructor
	 */
	public function __construct() {

		add_filter( 'user_registration_get_settings_pages', array( $this, 'add_invite_codes_setting' ), 10, 1 );
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_scripts' ), 20 );
		add_filter( 'manage_users_columns', array( $this, 'ur_add_code_column_head' ) );
		add_filter( 'manage_users_custom_column', array( $this, 'ur_add_code_column_cell' ), 20, 3 );

		// Frontend message settings.
		add_filter( 'user_registration_frontend_messages_settings', array( $this, 'add_invite_codes_frontend_message' ) );

		if ( current_user_can( 'manage_user_registration' ) ) {
			add_action( 'admin_print_scripts', array( $this, 'hide_unrelated_notices' ) );
		}
	}

	/**
	 * @param $settings
	 *
	 * @return array $settings
	 */
	public function add_invite_codes_setting( $settings ) {

		if ( class_exists( 'UR_Settings_Page' ) ) {
			$settings[] = include_once dirname( __FILE__ ) . '/admin/settings/class-user-registration-invite-codes-settings.php';
		}

		return $settings;
	}


	/**
	 * Enqueue scripts.
	 */
	public function admin_scripts() {
		$screen    = get_current_screen();
		$screen_id = $screen ? $screen->id : '';
		$suffix    = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

		if ( in_array( $screen_id, array( 'user-registration_page_user-registration-settings' ) ) ) {

			wp_enqueue_style( 'ur-invite-codes-admin-style', plugins_url( '/assets/css/user-registration-invite-codes-admin.css', UR_INVITE_CODES_PLUGIN_FILE ) );
			wp_register_script( 'ur-invite-codes-admin-script', plugins_url( '/assets/js/admin/user-registration-invite-codes-admin' . $suffix . '.js', UR_INVITE_CODES_PLUGIN_FILE ) );
			wp_enqueue_script( 'ur-invite-codes-admin-script' );
			wp_enqueue_script( 'flatpickr' );
			wp_enqueue_style( 'flatpickr' );
		}
	}

	/**
	 * Table list output.
	 */
	public static function invite_codes_list_table_output() {

		global $invite_codes_table_list;
		?>
		 <div class="wrap">
			<hr class="wp-header-end">

			<?php

				$forms    = ur_get_all_user_registration_form();
				$latest   = key( $forms );
				$selected = isset( $_REQUEST['form_id'] ) ? $_REQUEST['form_id'] : $latest;

			if ( isset( $_POST['form_id'] ) ) {
				$query_args = add_query_arg(
					array(
						'form_id' => $selected,
					),
					'//' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']
				);
				wp_safe_redirect( $query_args );
			}
			?>
				<form id="codes-select" method="POST">
					<select id = "form-select" name ="form_id">
						<?php
						foreach ( $forms as $key => $form ) {
							echo '<option value="' . $key . '" ' . selected( $selected, $key, false ) . '>' . $form . '</option>';
						}
						?>
					</select>
					<button type="submit" class="button" name ="select-form"><?php _e( 'Filter', 'user-registration-invite-codes' ); ?></button>
				</form>

			<?php
				$invite_codes_table_list = new UR_Invite_Codes_Table_List();
				$invite_codes_table_list->process_bulk_action();
				$invite_codes_table_list->prepare_items();
			?>

			<form id="codes-filter" method="post">
				<input type="hidden" name="page" value="ur-invite-codes">

				<?php
					$invite_codes_table_list->views();
					$invite_codes_table_list->search_box( __( 'Search Codes', 'user-registration-invite-codes' ), 'user-registration-invite-codes' );
					$invite_codes_table_list->display();
				?>

			</form>
		</div>
		<?php
	}

	/**
	 * Frontend message for wrong and invalid invite codes.
	 *
	 * @param  array $settings
	 * @return array $settings
	 */
	public function add_invite_codes_frontend_message( $settings ) {
		$paypal_settings = array(
			array(
				'title' => __( 'Invite Codes Messages', 'user-registration-invite-codes' ),
				'type'  => 'title',
				'desc'  => '',
				'id'    => 'invite_codes_messages_settings',
			),
			array(
				'title'    => __( 'Invalid Invite Code', 'user-registration-invite-codes' ),
				'desc'     => __( 'Enter the error message in form submission on invalid invite code.', 'user-registration-invite-codes' ),
				'id'       => 'user_registration_form_submission_error_message_invalid_invite_code',
				'type'     => 'text',
				'desc_tip' => true,
				'css'      => 'min-width: 350px;',
				'default'  => __( 'Invalid Invite Code. Contact site administrator.', 'user-registration-invite-codes' ),
			),
			array(
				'title'    => __( 'Expired Invite Code', 'user-registration-invite-codes' ),
				'desc'     => __( 'Enter the error message in form submission on expired invite code.', 'user-registration-invite-codes' ),
				'id'       => 'user_registration_form_submission_error_message_expired_invite_code',
				'type'     => 'text',
				'desc_tip' => true,
				'css'      => 'min-width: 350px;',
				'default'  => __( 'Invite Code Expired. Contact site administrator.', 'user-registration-invite-codes' ),
			),
			array(
				'title'    => __( 'User Limit Exceeded', 'user-registration-invite-codes' ),
				'desc'     => __( 'Enter the error message in form submission on exceeded user limit.', 'user-registration-invite-codes' ),
				'id'       => 'user_registration_form_submission_error_message_user_limit_exceeded_invite_code',
				'type'     => 'text',
				'desc_tip' => true,
				'css'      => 'min-width: 350px;',
				'default'  => __( 'User limit exceeded for this invite code. Contact site administrator.', 'user-registration-invite-codes' ),
			),
			array(
				'type' => 'sectionend',
				'id'   => 'invite_codes_messages_settings',
			),
		);

		$settings = array_merge( $settings, $paypal_settings );

		return $settings;
	}

	/**
	 * Add the column header
	 *
	 * @param array $columns
	 *
	 * @return array
	 */
	public function ur_add_code_column_head( $columns ) {

		if ( ! current_user_can( 'edit_user' ) ) {
			return $columns;
		}

		$columns['ur_invite_code'] = __( 'Invite Code', 'user-registration-invite-codes' );
		return $columns;
	}

	/**
	 * Set the Column value for each user in the users list
	 *
	 * @param string $val
	 * @param string $column_name
	 * @param int    $user_id
	 *
	 * @return string
	 */
	public function ur_add_code_column_cell( $value, $column_name, $user_id ) {

		if ( $column_name === 'ur_invite_code' ) {

			$value = '-';

			// Get form ID.
			$form_id = get_user_meta( $user_id, 'ur_form_id', true );

			$post = get_post( $form_id );

			if ( get_post_type( $post ) !== 'user_registration' ) {
				return $value;
			}

			$post_content       = isset( $post->post_content ) ? $post->post_content : '';
			$post_content_array = json_decode( $post_content );

			foreach ( $post_content_array as $post_content_row ) {
				foreach ( $post_content_row as $post_content_grid ) {
					foreach ( $post_content_grid as $field ) {
						if ( isset( $field->field_key ) && isset( $field->general_setting->field_name ) ) {
							$invite_code_field = $field->general_setting->field_name;
						}
					}
				}
			}

			// Return if invite code is not set.
			if ( ! $invite_code_field ) {
				return $value;
			}

			$value = get_user_meta( $user_id, 'user_registration_' . $invite_code_field, true );

			return $value;
		}

		return $value;
	}

	/**
	 * Hide Notices From WPList table for invite codes section.
	 * For Clear appearance.
	 *
	 * @return void
	 */
	public function hide_unrelated_notices() {
		global $wp_filter;

		// Return on other than user registraion builder page.
		if ( empty( $_REQUEST['page'] ) || 'user-registration-settings' !== $_REQUEST['page'] || empty( $_REQUEST['tab'] ) || 'user-registration-invite-codes' !== $_REQUEST['tab'] ) {
			return;
		}

		foreach ( array( 'user_admin_notices', 'admin_notices', 'all_admin_notices' ) as $wp_notice ) {
			if ( ! empty( $wp_filter[ $wp_notice ]->callbacks ) && is_array( $wp_filter[ $wp_notice ]->callbacks ) ) {
				foreach ( $wp_filter[ $wp_notice ]->callbacks as $priority => $hooks ) {
					foreach ( $hooks as $name => $arr ) {
						unset( $wp_filter[ $wp_notice ]->callbacks[ $priority ][ $name ] );
					}
				}
			}
		}
	}
}

new User_Registration_Invite_Codes_Admin();
