<?php
/**
 * User_Registration_Invite_Codes setup
 *
 * @package User_Registration_Invite_Codes
 * @since   1.0.0
 */

defined( 'ABSPATH' ) || exit;

/**
 * Main User_Registration_Invite_Codes Class.
 *
 * @class User_Registration_Invite_Codes
 */
final class User_Registration_Invite_Codes {

	/**
	 * Plugin version.
	 *
	 * @var string
	 */
	const VERSION = '1.0.1';

	/**
	 * Instance of this class.
	 *
	 * @var object
	 */
	protected static $instance = null;

	/**
	 * Initialize the plugin.
	 */
	private function __construct() {
		// Load plugin text domain.
		add_action( 'init', array( $this, 'load_plugin_textdomain' ) );

		// Checks if user registration is installed
		if ( defined( 'UR_VERSION' ) && version_compare( UR_VERSION, '1.5.3', '>=' ) ) {
			$this->includes();
			add_action( 'user_registration_init', array( $this, 'plugin_updater' ) );
		} else {
			add_action( 'admin_notices', array( $this, 'user_registation_missing_notice' ) );
		}
	}

	/**
	 * Return an instance of this class.
	 *
	 * @return object A single instance of this class.
	 */
	public static function get_instance() {
		// If the single instance hasn't been set, set it now.
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	/**
	 * Load Localisation files.
	 *
	 * Note: the first-loaded translation file overrides any following ones if the same translation is present.
	 *
	 * Locales found in:
	 *      - WP_LANG_DIR/user-registration-invite-codes/user-registration-invite-codes-LOCALE.mo
	 *      - WP_LANG_DIR/plugins/user-registration-invite-codes-LOCALE.mo
	 */
	public function load_plugin_textdomain() {
		$locale = apply_filters( 'plugin_locale', get_locale(), 'user-registration-invite-codes' );

		load_textdomain( 'user-registration-invite-codes', WP_LANG_DIR . '/user-registration-invite-codes/user-registration-invite-codes-' . $locale . '.mo' );
		load_plugin_textdomain( 'user-registration-invite-codes', false, plugin_basename( dirname( UR_INVITE_CODES_PLUGIN_FILE ) ) . '/languages' );
	}

	/**
	 * Plugin Updater.
	 */
	public function plugin_updater() {
		if ( function_exists( 'ur_addon_updater' ) ) {
			ur_addon_updater( UR_INVITE_CODES_PLUGIN_FILE, 9157, self::VERSION );
		}
	}

	/**
	 * Includes.
	 */
	private function includes() {

		include_once dirname( __FILE__ ) . '/functions-invite-codes-handler.php';

		if ( ! class_exists( 'UR_Form_Field_Invite_Code' ) ) {
			include_once dirname( __FILE__ ) . '/form/class-ur-form-field-invite-code.php';
		}

		if ( is_admin() ) {
			include_once dirname( __FILE__ ) . '/class-user-registration-invite-codes-admin.php';
			include_once dirname( __FILE__ ) . '/class-user-registration-invite-codes-list-table.php';
		}
	}

	/**
	 * User Registration fallback notice.
	 */
	public function user_registation_missing_notice() {
		/* translators: %s: user-registration plugin link */
		echo '<div class="error notice is-dismissible"><p>' . sprintf( esc_html__( 'User Registration Invite Codes requires %s version 1.5.4 or greater to work!', 'user-registration-invite-codes' ), '<a href="https://wpeverest.com/wordpress-plugins/user-registration/" target="_blank">' . esc_html__( 'User Registration', 'user-registration-invite-codes' ) . '</a>' ) . '</p></div>';
	}
}
