<?php
/**
 * UserRegistrationInviteCode Hander Functions.
 *
 * General core functions available on both the front-end and admin.
 *
 * @package UserRegistrationInviteCodes
 *
 * @version 1.0.0
 */

defined( 'ABSPATH' ) || exit;

add_action( 'admin_init', 'ur_invite_codes_actions' );

// Check for valid invite codes on frontend.
add_filter( 'user_registration_response_array', 'ur_invite_codes_response', 10, 3 );

// Override user role.
add_filter( 'user_registration_user_role', 'ur_invite_codes_user_role', 10, 3 );

// Register Invite Code field.
add_filter( 'user_registration_registered_form_fields', 'ur_register_invite_code_field' );

// Include Invite Code field.
add_filter( 'user_registration_form_field_invite_code_path', 'ur_include_invite_code_field', 10, 1 );

add_filter( 'user_registration_invite_code_admin_template', 'ur_add_invite_code_template', 10, 1 );

add_filter( 'user_registration_sanitize_field', 'ur_sanitize_invite_code_field', 10, 2 );

add_filter( 'user_registration_one_time_draggable_form_fields', 'ur_invite_code_one_time_drag', 10, 1 );

/**
 * Handles all settings action.
 *
 * @return void.
 */
function ur_invite_codes_settings_handler() {

	if ( ! empty( $_POST ) ) {

		// Nonce Check.
		if ( empty( $_REQUEST['_wpnonce'] ) || ! wp_verify_nonce( $_REQUEST['_wpnonce'], 'user-registration-settings' ) ) {
			die( __( 'Action failed. Please refresh the page and retry.', 'user-registration-invite-codes' ) );
		}

		// Update the codes for add new functionality
		if ( ( isset( $_POST['user_registration_add_new_code'] ) && ! empty( $_POST['user_registration_add_new_code'] ) ) || ( isset( $_POST['user_registration_edit_code'] ) && ! empty( $_POST['user_registration_edit_code'] ) ) ) {

			$code        = isset( $_POST['user_registration_add_new_code'] ) ? $_POST['user_registration_add_new_code'] : $_POST['user_registration_edit_code'];
			$user_limit  = isset( $_POST['user_registration_invite_codes_user_limit'] ) ? $_POST['user_registration_invite_codes_user_limit'] : '';
			$expiry_date = isset( $_POST['user_registration_invite_codes_expiry_date'] ) ? $_POST['user_registration_invite_codes_expiry_date'] : '';
			$form        = isset( $_POST['user_registration_invite_codes_select_form'] ) ? $_POST['user_registration_invite_codes_select_form'] : '';
			$role        = isset( $_POST['user_registration_invite_codes_user_role'] ) ? $_POST['user_registration_invite_codes_user_role'] : '';
			$option      = get_option( 'ur_invite_codes', array() );

			if ( empty( $option ) ) { // In case if option is set but empty.
				$option = array();
			}

			$user_count = isset( $option[ $code ]['user_count'] ) ? $option[ $code ]['user_count'] : '';

			$option[ $code ] = array(
				'user_limit'   => $user_limit,
				'user_count'   => $user_count,
				'created_date' => date( 'Y-m-d' ),
				'expiry_date'  => $expiry_date,
				'form'         => $form,
				'role'         => $role,
			);

			if ( isset( $_REQUEST['editing'] ) && $_REQUEST['editing'] !== $code ) {

				$editing = $_REQUEST['editing'];
				unset( $option[ $editing ] );
			}

			update_option( 'ur_invite_codes', $option );

			return true;
		}

		// Generate Random Codes.
		if ( isset( $_POST['user_registration_invite_codes_random_number'] ) && ! empty( $_POST['user_registration_invite_codes_random_number'] ) ) {
			$number        = absint( $_POST['user_registration_invite_codes_random_number'] );
			$prefix        = isset( $_POST['user_registration_invite_codes_random_codes_prefix'] ) ? $_POST['user_registration_invite_codes_random_codes_prefix'] : '';
			$r_user_limit  = isset( $_POST['user_registration_invite_codes_random_user_limit'] ) ? $_POST['user_registration_invite_codes_random_user_limit'] : '';
			$r_expiry_date = isset( $_POST['user_registration_invite_codes_random_expiry_date'] ) ? $_POST['user_registration_invite_codes_random_expiry_date'] : '';
			$r_form        = isset( $_POST['user_registration_invite_codes_random_select_form'] ) ? $_POST['user_registration_invite_codes_random_select_form'] : '';
			$r_role        = isset( $_POST['user_registration_invite_codes_random_user_role'] ) ? $_POST['user_registration_invite_codes_random_user_role'] : '';
			$option        = get_option( 'ur_invite_codes', array() );

			if ( empty( $option ) ) { // In case if option is set but empty.
				$option = array();
			}

			for ( $times = 1; $times <= $number;  $times++ ) {

				$random_letters                      = substr( str_shuffle( 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' ), 0, 5 );
				$option[ $prefix . $random_letters ] = array(
					'user_limit'   => $r_user_limit,
					'user_count'   => '',
					'created_date' => date( 'Y-m-d' ),
					'expiry_date'  => $r_expiry_date,
					'form'         => $r_form,
					'role'         => $r_role,
				);

				update_option( 'ur_invite_codes', $option );
			}

			return true;
		}
	}
}

/**
 * All Invite Codes Actions.
 *
 * @return void
 */
function ur_invite_codes_actions() {

	// Delete Single Code.
	if ( isset( $_GET['action'], $_GET['delete_code'] ) ) {
		ur_delete_code();
	}

	// Bulk Delete Action.
	if ( isset( $_REQUEST['page'] ) && $_REQUEST['page'] === 'ur-invite-codes' && isset( $_REQUEST['action'] ) && $_REQUEST['action'] === 'delete' ) {
		ur_bulk_delete_codes();
	}

	// Edt Code.
	if ( isset( $_GET['action'], $_GET['edit_code'] ) ) {
		ur_edit_code();
	}
}

/**
 * Delete single code.
 *
 * @return void
 */
function ur_delete_code() {
	check_admin_referer( 'delete_code' );

	if ( ! current_user_can( 'delete_posts' ) ) {
		return;
	}

	$code    = $_GET['delete_code'];
	$options = get_option( 'ur_invite_codes', array() );

	unset( $options[ $code ] );

	update_option( 'ur_invite_codes', $options );

	$status = isset( $_REQUEST['status'] ) ? $_REQUEST['status'] : 'all';

	wp_redirect( esc_url_raw( add_query_arg( array( 'deleted' => 1 ), admin_url( 'admin.php?page=user-registration-settings&tab=user-registration-invite-codes&status=' . $status ) ) ) );
	exit();
}


/**
 * Delete single code.
 *
 * @return void
 */
function ur_edit_code() {

	check_admin_referer( 'edit_code' );

	if ( ! current_user_can( 'edit_posts' ) ) {
		return;
	}

	$code = $_REQUEST['edit_code'];

	wp_redirect( esc_url_raw( add_query_arg( array( 'editing' => $code ), admin_url( 'admin.php?page=user-registration-settings&tab=user-registration-invite-codes&section=edit-code' ) ) ) );

	exit();
}

/**
 * Search invite codes.
 *
 * @param array $items All Codes
 *
 * @return array All searched codes.
 */
function ur_invite_codes_search( $items ) {

	// Verify Nonce.
	check_admin_referer( 'bulk-invite-codes' );

	$search  = $_REQUEST['s'];
	$s_items = array();

	foreach ( $items as $item ) {

		if ( empty( $item['code'] ) ) {
			continue;
		}

		if ( strpos( $item['code'], $search ) !== false ) {
			$s_items[] = $item;
		}
	}

	return $s_items;    // Searched Items.
}

/**
 * Process Bulk Delete Action
 *
 * @return void
 */
function ur_bulk_delete_codes() {

	// Verify nonce.
	check_admin_referer( 'bulk-invite-codes' );

	if ( isset( $_REQUEST['invite-code'] ) ) {
		$d_codes = $_REQUEST['invite-code']; // Codes to delete.

		$options = get_option( 'ur_invite_codes', array() );

		foreach ( $d_codes as $code ) {
			if ( ! isset( $options[ $code ] ) ) {
				continue;
			}
			unset( $options[ $code ] );
		}

		update_option( 'ur_invite_codes', $options );

		$status = isset( $_REQUEST['status'] ) ? $_REQUEST['status'] : 'all';
		wp_redirect( esc_url_raw( add_query_arg( array( 'deleted' => true ), admin_url( 'admin.php?page=user-registration-settings&tab=user-registration-invite-codes&status=' . $status ) ) ) );
		exit();
	}
}

/**
 * Checks for valid invite codes.
 *
 * @param array $response   Response of the form.
 * @param array $form_Data  Form Data.
 * @param int   $form_id    Form ID.
 *
 * @return mixed
 */
function ur_invite_codes_response( $response, $form_data, $form_id ) {

	$invite_code_field = false;

	// Get invite code field value;
	foreach ( $form_data as $data ) {

		if ( $data->extra_params['field_key'] === 'invite_code' ) {
			$invite_code_value = $data->value;
			$invite_code_field = true;
			break;
		}
	}

	// Return if invite code is not set.
	if ( ! $invite_code_field ) {
		return $response;
	}

	$codes = get_option( 'ur_invite_codes', array() );

	foreach ( $codes as $code => $options ) {

		// Code matches.
		if ( strval( $code ) === strval( $invite_code_value ) && $options['form'] == $form_id ) {

			$current_date          = date( 'Y-m-d' );
			$expiry_date           = isset( $options['expiry_date'] ) ? $options['expiry_date'] : '';
			$user_limit            = isset( $options['user_limit'] ) ? $options['user_limit'] : '';
			$user_count            = isset( $options['user_count'] ) ? $options['user_count'] : '';
			$options['user_count'] = isset( $options['user_count'] ) ? absint( $options['user_count'] ) : 0;

			// Check for invite code validity.
			if ( ! isset( $expiry_date ) || empty( $expiry_date ) ) {

				// Check for user limit.
				$response = ur_check_user_limit( $user_limit, $user_count, $response, $code );
				return $response;

			} // Expired invite codes.
			elseif ( $current_date > $expiry_date ) {
				array_push( $response, get_option( 'user_registration_form_submission_error_message_expired_invite_code', __( 'Invite Code Expired. Contact site administrator.', 'user-registration-invite-codes' ) ) );

				// Check for user limit.
				$response = ur_check_user_limit( $user_limit, $user_count, $response, $code );
				return $response;

			} else {

				// Check for user limit.
				$response = ur_check_user_limit( $user_limit, $user_count, $response, $code );
				return $response;
			}
		}
	}

	array_push( $response, get_option( 'user_registration_form_submission_error_message_invalid_invite_code', __( 'Invalid Invite Code. Contact site administrator.', 'user-registration-invite-codes' ) ) );

	return $response;
}

/**
 * Checks for user limit
 *
 * @param  int   $user_limit Number of users that can be registered with this invite code.
 * @param  int   $user_count Number of already registered users with this invite code.
 * @param  array $response
 *
 * @return mixed.
 */
function ur_check_user_limit( $user_limit, $user_count, $response, $code_value ) {

	// Check for user limit.
	if ( ! empty( $user_limit ) && ! empty( $user_count ) ) {
		if ( $user_limit <= $user_count ) {
			array_push( $response, get_option( 'user_registration_form_submission_error_message_user_limit_exceeded_invite_code', __( 'User limit exceeded for this invite code. Contact site administrator.', 'user-registration-invite-codes' ) ) );
		}
	}

	// Update user count on user limit check pass.
	if ( count( $response ) <= 0 ) {
		$codes   = get_option( 'ur_invite_codes', array() );
		$u_codes = array();

		foreach ( $codes as $code => $options ) {

			if ( $code === $code_value ) {
				$options['user_count']++;

			}

			$u_codes[ $code ] = $options;

			update_option( 'ur_invite_codes', $u_codes );
		}
	}

	return $response;
}

/**
 * Assign user role with invite code.
 *
 * @param  string $user_role User role in form settings.
 * @param  array  $form_data Form Data.
 * @param  int    $form_id   Form ID.
 * @return string $user_role User role with invitation code.
 */
function ur_invite_codes_user_role( $user_role, $form_data, $form_id ) {

	$invite_code_field = false;

	// Get invite code field value;
	foreach ( $form_data as $data ) {

		if ( $data->extra_params['field_key'] === 'invite_code' ) {
			$invite_code_value = $data->value;
			$invite_code_field = true;
			break;
		}
	}

	// Return if invite code is not set.
	if ( ! $invite_code_field ) {
		return $user_role;
	}

	$codes = get_option( 'ur_invite_codes', array() );

	foreach ( $codes as $code => $options ) {

		// Code matches.
		if ( $code === $invite_code_value && $options['form'] == $form_id ) {
			$user_role = $options['role'];
			return $user_role;
		}
	}

	return $user_role;
}

/**
 * Register invite code field.
 *
 * @return array  All registered fields.
 */
function ur_register_invite_code_field( $fields ) {
	$fields[] = 'invite_code';
	return $fields;
}

/**
 * Include Invite Code Field.
 *
 * @param $path
 *
 * @return string
 */
function ur_include_invite_code_field( $path ) {

	include_once dirname( __FILE__ ) . '/form/class-ur-form-field-invite-code.php';
}

/**
 * Include template for invite code field.
 *
 * @return string   File Path.
 */
function ur_add_invite_code_template( $path ) {

	$path = dirname( __FILE__ ) . '/form/views/admin/admin-invite-code.php';

	return $path;
}

/**
 * Sanitize Invite Code fields on frontend submit
 *
 * @param  array  $form_data    Form Data.
 * @param  string $field_key    Field Key.
 * @return array  Form Data.
 */
function ur_sanitize_invite_code_field( $form_data, $field_key ) {

	switch ( $field_key ) {
		case 'invite_code':
			if ( ! empty( $form_data->value ) ) {
				$form_data->value = sanitize_text_field( $form_data->value );
			}

			break;
	}

	return $form_data;
}

/**
 * Make invite code one time draggable.
 *
 * @param array     One time draggable fields.
 *
 * @return array    One time draggable fields.
 */
function ur_invite_code_one_time_drag( $fields ) {
	$fields[] = 'invite_code';
	return $fields;
}
